// CompoundAssignment.do
version 11

/* *********************************************************************
* PART 0: HOUSEKEEPING
********************************************************************* */
clear all
mata: mata clear
cap log close
set more off
set trace off
set linesize 100
set memory 200m
set matsize 800
set scheme s2mono
set seed 123456789
timer clear

cap cd "C:\Users\New User\Dropbox\Work\Technical Note\Shared\sequential\toms-directory\Stata\implementation"

local LOGNAME daily-compound-assignment
log using "logfiles/`LOGNAME'_$S_DATE", append
display "$S_DATE"
pwd

timer on 100
pause on



/*
	CompoundAssignment.do

	A user interface for Atkinson's Deterministic
		treatment assignment

		
 	PART 0.A: INTRODUCTION AND DISCUSSION
	
	descriptive statistics, tables and figures
	
	inputs: real data (compound id and covariates 
			(compound size and presence of gas))

	output:	log file and various data files located in 
		data/view, data/use, and data/safe
				
	Version history:

		20110903	TP		first version "CompoundAssignment-20110903"		

	Directory structure
	
		base:				Dropbox/Cholera-Disgust/Randomization
		subdirectories:		/data			
							/data/use		data which will be used by the do-file in subsequent runs (not to be touched)
							/data/view		data for viewing
							/data/safe		a log of important data by date and entry number (not to be touched)
							/data/raw		source data (not to be touched)
							/dofiles		contains this and other dofiles
							/logfiles		save log files here
							/figures		export figures (eps, pdf) here
							/tables			tables created here
												contains excel and pdf
							/tables/txt		export raw material for tables here
												contains txt and csv
*/


/* ********************************************************************
* PART 0.B:	Define useful parameters (macros, scalars, etc), labels, etc
********************************************************************* */
	
/* *********************************************************************
* PART 0.C:	Define programs
********************************************************************* */
// programs here

/* *********************************************************************
* PART 0.D:	Sections to run / skip
********************************************************************* */



//Generate counter for labelling files
//If entering more than once in a day (up to 100 times), then this code will catch and change the index of the data set from 1 to the correct number.
forvalues n=1/100 {
capture confirm file "data/safe/TodaysCovariates_`n'_$S_DATE.dta"
	if _rc==601 {
		local num `n'
		continue, break
		}		
	}
		//
		
		
//Generate table for entering data
set obs 1
gen Comp_id=.
gen num_hh=.
gen byte gas=.


//Prompt user to enter the data.
display "FIRST ENTRY: Enter today's data in the data editor. To begin, type 'q' in the command line and press enter. When finished please type 'end' in the command line and press enter."
pause
edit

//After the user has entered data, he will type "end" and press enter. Then the first entry will be saved, the table cleared, and prepared for 
//second entry. 
pause

//Before saving, generate the variable line_num which gives the line number of each set of compound covariates (for use in fixing errors below)
gen line_num=_n
save data/use/entry1, replace
clear



//Generate table for reentry
set obs 1
gen Comp_id=.
gen num_hh=.
gen byte gas=.

//Prompt user to enter the data.
display "SECOND ENTRY: Reenter today's data in the data editor. To begin, type 'q' in the command line and press enter. When finished please type 'end' in the command line and press enter."
pause
edit
pause

gen line_num=_n
save data/use/entry2, replace
clear





//Now compare the two entries
use data/use/entry1, clear
merge 1:1  Comp_id num_hh gas line_num using data/use/entry2
sum _m


//If the two entries are identical, then save one dataset randomize the order of the entries and proceed to the assignment do-file.
if r(mean)==3 {
drop _merge
save "data/safe/TodaysCovariates_`num'_$S_DATE"
save data/use/TodaysCovariates_`num', replace
erase "data/use/entry1.dta"
erase "data/use/entry2.dta"
gen randnum=runiform()
sort randnum
drop randnum line_num
do "dofiles/Deterministic-Daily-20110906-tp.do"

}
//

//If they are not identical, then notify the user and allow him or her to view and fix the discrepancies (Users are given two opportunities to 
//resolve discrepancies after which they are asked to start from the beginning and reenter carefully)
else{
drop if _m==3
gen Entry="Entry 1"
replace Entry="Entry 2" if _m==2
sort line_num Entry
order line_num Entry
drop _m
display "There is some discrepancy between the two entries. Please fix the mistakes in the data editor, then type 'end' in the command line and press enter. To begin, type 'q' in the command line and press enter. If it is unclear how to fix the mistakes in the data editor, then please start over and carefully reenter the data."
pause
edit
pause
save "data/use/fix"
drop if Entry=="Entry 2"
drop Entry
save "data/use/fix1"

use "data/use/fix", clear
drop if Entry=="Entry 1"
drop Entry
save "data/use/fix2"

use data/use/entry1, clear
merge 1:1  line_num using "data/use/fix1", replace update
drop _m
save data/use/entry1, replace

use data/use/entry2, clear
merge 1:1  line_num using "data/use/fix2", replace update
drop _m
save data/use/entry2, replace

erase "data/use/fix.dta"
erase "data/use/fix1.dta"
erase "data/use/fix2.dta"




//Now compare the two entries AGAIN
use data/use/entry1, clear
merge 1:1  Comp_id num_hh gas line_num using data/use/entry2
sum _m

//If they're good move on
if r(mean)==3 {
drop _merge
save "data/safe/TodaysCovariates_`num'_$S_DATE"
save data/use/TodaysCovariates_`num', replace
erase "data/use/entry1.dta"
erase "data/use/entry2.dta"
gen randnum=runiform()
sort randnum
drop randnum line_num
do "dofiles/Deterministic-Daily-20110906-tp.do"

}
//If not good, give them the second chance
else{
drop if _m==3
gen Entry="Entry 1"
replace Entry="Entry 2" if _m==2
sort line_num Entry
order line_num Entry
drop _m
display "There is STILL some discrepancy between the two entries. Please fix the mistakes in the data editor, then type 'end' in the command line and press enter. To begin, type 'q' in the command line and press enter. If it is unclear how to fix the mistakes in the data editor, then please start over and carefully reenter the data."
pause
edit
pause
save "data/use/fix"
drop if Entry=="Entry 2"
drop Entry
save "data/use/fix1"

use "data/use/fix", clear
drop if Entry=="Entry 1"
drop Entry
save "data/use/fix2"

use data/use/entry1, clear
merge 1:1  line_num using "data/use/fix1", replace update
drop _m
save data/use/entry1, replace

use data/use/entry2, clear
merge 1:1  line_num using "data/use/fix2", replace update
drop _m
save data/use/entry2, replace

erase "data/use/fix.dta"
erase "data/use/fix1.dta"
erase "data/use/fix2.dta"

//Now compare the two entries ONE LAST TIME
use data/use/entry1, clear
merge 1:1  Comp_id num_hh gas line_num using data/use/entry2
sum _m

//If they're good move on
if r(mean)==3 {
drop _merge
save "data/safe/TodaysCovariates_`num'_$S_DATE"
save data/use/TodaysCovariates_`num', replace
erase "data/use/entry1.dta"
erase "data/use/entry2.dta"
gen randnum=runiform()
sort randnum
drop randnum line_num
do "dofiles/Deterministic-Daily-20110906-tp.do"

}
//If still not good, ask to start over. 
else{
display "The discrepancy is STILL not resolved. Please start over and enter the data very carefully. Clear all, reopen and run the do-file."
}

}
//


}
//

************************************************************************
* END
************************************************************************
mata
mata clear
end
timer off 100
timer list
cap log close
exit
