// Deterministic-Daily.do
//version 11

/* *********************************************************************
* PART 0: HOUSEKEEPING
********************************************************************* */
//not necessary since it's already done in CompoundAssignment.do



/*
	Deterministic_Daily.do

 	PART 0.A: INTRODUCTION AND DISCUSSION
	
	descriptive statistics, tables and figures
	
	inputs: Compound IDs and covariates from CompoundAssignment.do

	output:	log file and various data files located in 
		data/view, data/use, and data/safe
				
	Version history:

		20110807	RPG		first version "simple-examples.do"
		20110808	RPG		more systematic procedure for first units
							tiebreaker
		20110814	RPG		rename "atkinson-deterministic-example-standalone.do"
		20110814	RPG		rename "atkinson-deterministic-weighted.do"
							add weighting elements
		20110903	TP		first version "Deterministic-Daily.do"
		

	Directory structure
	
		base:				Dropbox/Cholera-Disgust/Randomization
		subdirectories:		/data			
							/data/use		data which will be used by the do-file in subsequent runs (not to be touched)
							/data/view		data for viewing
							/data/safe		a log of important data by date and entry number (not to be touched)
							/data/raw		source data (not to be touched)
							/dofiles		contains this and other dofiles
							/logfiles		save log files here
							/figures		export figures (eps, pdf) here
							/tables			tables created here
												contains excel and pdf
							/tables/txt		export raw material for tables here
												contains txt and csv
*/

/* ********************************************************************
* PART 0.B:	Define useful parameters (macros, scalars, etc), labels, etc
********************************************************************* */
	
/* *********************************************************************
* PART 0.C:	Define programs
********************************************************************* */
// programs here

/* *********************************************************************
* PART 0.D:	Sections to run / skip
********************************************************************* */

//set up constants
local NUM_TREATMENTS		8
local NUMBER_TO_OVERWEIGHT	4
local INFLATION_FACTOR		= sqrt(5)



//If entering three times in a day, then this code will catch and change the index of the dataset to the correct number
forvalues n=1/100 {
	capture confirm file "data/use/TodaysCovariates_`n'.dta"
	if _rc==0 {
		local num `n'
		continue, break
		}
	}
//
	

	// PART A: read in key parameters
	//	set up some parameters and other inputs that do not
	//	change from observation to observation

mata:
	// number of treatments (t in Atk notation)
	NUM_TREATMENTS = strtoreal(st_local("NUM_TREATMENTS"))
	NUM_TREATMENTS	
	
	
	NUMBER_TO_OVERWEIGHT = strtoreal(st_local("NUMBER_TO_OVERWEIGHT"))
	NUMBER_TO_OVERWEIGHT
	
	INFLATION_FACTOR = strtoreal(st_local("INFLATION_FACTOR"))
	INFLATION_FACTOR
	
	// an identity matrix with one row per possible treatment
	POTENTIAL_TREATMENTS = I(NUM_TREATMENTS)
	POTENTIAL_TREATMENTS
	
	
	// get all the newly entered covariates
	st_view(DATA=.,.,("num_hh","gas"))
	//DATA[1..2,1..2]
	NUM_COVARS = cols(DATA)
	NUM_COVARS
	
	// number of observations
	NUM_OBS = rows(DATA)
	NUM_OBS
	// note that NUM_COVARS = q-1 in Atkinson's notation

		
	// initialize variables that will contain index of max-var treatment
	//The command: maxindex(A,num,ii,w) will set and save ii and w
		//ii lists the indices in A where the maxima (of A) are located
		//w lists first the index in ii where a list of indices for one maximum begins, then it lists the number (quantity) of that maxima that A contains.
	ii = J(0,0,.)
	w  = J(0,0,.)

	// similarly, the matrix of contrasts (L' in Atk) and
	//	the A matrix (L augmented with a block of zeros)
	//	do not change from one observation to the next

	// L': matrix of contrasts
	L_prime_1 = J(NUM_TREATMENTS-1,1,1)
	L_prime_2 = I(NUM_TREATMENTS-1,NUM_TREATMENTS-1)
	L_prime = L_prime_1, (-1*L_prime_2)
	L_prime
	L = L_prime'
	L
	L'
	
	// A': augment with block of zeros to zero out the nuisance parameters
	A_prime_2 = J(NUM_TREATMENTS-1,NUM_COVARS,0)
	A_prime = L', A_prime_2
	A = A_prime'
	A
	A'
	
	
	// record dimensions of various important matrices
	num_cols_H = NUM_TREATMENTS
	num_cols_Z = NUM_COVARS
	num_cols_G = num_cols_H + num_cols_Z
	dim_GprimeG = num_cols_G						// GprimeG def'd below
	num_cols_H, num_cols_Z, num_cols_G, dim_GprimeG
	
	
	// need some noise to regularize information matrix for first t
	//	observations (see discussion below, lines 167-168)
	epsilon = 0.001
	diagonal_epsilon = epsilon*I(dim_GprimeG,dim_GprimeG)
	
		
	end 
	
	
	
	
	// PART B: treatment assignment
	// for the first NUM_TREATMENTS (t in Atk 2001) observations,
	//	the information matrix G'G is singular, so we cannot use
	//	the Atkinson method without some adjustment
	//	Atkinson (1999, pg 1749; 2001, g 371) recommends adding some
	//	noise epsilon (he used 0.001) to the diagonal of G'G
	// an alternative would be to use a simpler randomization strategy
	//	for the first t observations, e.g. simple randomization, some
	//	stratified randomization, or minimization
	
	// **** B.1 FIRST OBSERVATION ****

	
	//Check whether this is the first assigment by checking whether the Compoundassignment data set has been created
	capture confirm file "data\use\Compoundassignment.dta"
	
	
	//If it's the first assignment, randomly assign the first treatment and create the G matrix
	//Otherwise open the previously saved Matrices and get on with randomizing the list of compounds

	if _rc==601 {

	//set this local to be used in the loop below (start with 2 if first time through, and 1 all times after)
	local s 2

	mata
	
	//If first time, initialize these storage vectors/matrices
	treatment_history 	= J(NUM_OBS,1,.)
	d_A_history 		= J(NUM_OBS,NUM_TREATMENTS,.)
	efficiency			= J(NUM_OBS,1,.)
	loss 				= J(NUM_OBS,1,.)

	
	// matrix of treatment assignments H
	H = J(1,NUM_TREATMENTS,0)
	// assign first at random
	first_treatment = ceil((NUM_TREATMENTS-1+1)*runiform(1,1))
	H[1,first_treatment] = 1
	H
	
	treatment_history[1,1] = first_treatment
	
	// matrix of covariates Z
	Z = DATA[|1,1 \ 1,.|]
	Z
	
	G = H, Z		// this is G_n in Atk notation
	G
	
	end
	}	
	else {

	//set this local to be used in the loop below (start with 2 if first time through, and 1 all times after)
		local s 1
	
		mata
		//bring in the matrix G 
		mata matuse "data/use/Matrices"
		
		//Reset storage matrices
		treatment_history 	= J(NUM_OBS,1,.)
		d_A_history 		= J(NUM_OBS,NUM_TREATMENTS,.)
		efficiency			= J(NUM_OBS,1,.)
		loss 				= J(NUM_OBS,1,.)	
		
		end
	}

	//Proceed to loop through the newly entered observations and assign them accordingly
	
	mata
	
	
	for (n=`s'; n<=NUM_OBS; n++) {
		printf("Observation %f\n", n)
			
			
		GprimeG = G'*G
		// GprimeG
		// det(GprimeG)
		
		if (det(GprimeG)==0) {
			printf("GprimeG matrix is singular, adding epsilon noise\n")
			GprimeG = GprimeG + diagonal_epsilon
		}
		
		// holds values of generalized variance 
		d_A 	= J(1,NUM_TREATMENTS,.)
		
		z_nplus1 = (DATA[|n,1 \ n,.|])'

		for (j=1; j<=NUM_TREATMENTS; j++) {
			h_nplus1 = (POTENTIAL_TREATMENTS[j,.])'
			g_nplus1 = h_nplus1 \ z_nplus1
			// g_nplus1

			d_A_j = g_nplus1'*invsym(GprimeG)*A*invsym(A'*invsym(GprimeG)*A)*A'invsym(GprimeG)*g_nplus1
			// d_A_j
			d_A[1,j] = d_A_j
				
		}
		
		// inflate the variance of treatments we wish to over-weight
		// we over-weight the first (NUMBER_TO_OVERWEIGHT), so columns 1 - NUMBER_TO_OVERWEIGHT
		d_A[|1,1 \ 1,NUMBER_TO_OVERWEIGHT|]  = INFLATION_FACTOR*d_A[|1,1 \ 1,NUMBER_TO_OVERWEIGHT|]		
		d_A_history[n,] = d_A
		
		
		// d_A
		// have to round these off to float precision
		//	otherwise numbers that are equal will appear not to be
		maxindex(floatround(d_A), 1, ii, w)
		// ii
		// w

		h_nplus1 = J(NUM_TREATMENTS,1,0)
		// if there is a unique maximum, use that one
		if (rows(ii)==1) {
			h_nplus1[ii,1] = 1
			treatment_history[n,1] = ii
		}
		else {
			printf("%f treatments tied, choosing randomly among these\n",rows(ii)) 
			// randomize among maxima
			which_of_ii = ceil((rows(ii)-1+1)*runiform(1,1))
			choice = ii[which_of_ii,1]
			h_nplus1[choice,1] = 1
			treatment_history[n,1] = choice
		}
		
		g_nplus1 = h_nplus1 \ z_nplus1
		// g_nplus1
		
		/*= H \ (h_nplus1')
		Z = Z \ (z_nplus1')
		G = H, Z
		*/
		G = G \ (g_nplus1')
		// G
	
		
		efficiency[n,1] = ( ( (NUM_TREATMENTS^NUM_TREATMENTS) / (n^(NUM_TREATMENTS-1))) ///
			/ (det(A'*invsym(G'*G)*A)) ///
			) ///
			^ (1 / (NUM_TREATMENTS-1))
			
	}
	
	
	// add treatment, efficiency and loss histories to stata dataset
	treatment_index = st_addvar("byte","treatment")
	st_store((1,rows(treatment_history)),treatment_index,treatment_history)
	efficiency_index = st_addvar("float","efficiency")
	st_store((1,rows(efficiency)),efficiency_index,efficiency)
	
	end
	
	
	
	/*	
	gen loss = _n * (1 - efficiency)
		Needs to be coded so _n takes off where it left off in previous entry

	*/
	
	
	//This bit of code displays some information about the data so you can check to see if the program is working properly 
	//It finds the mean compound size and generates a binary variable for greater than mean or smaller than mean. It will be useful when we have 
	//more data, but for now it doesn't have much meaning. 
	/*
	tab treatment
	tab treatment gas, miss
	summ num_hh, det
	gen byte large_compound = num_hh>`r(p50)'
	tab treatment large_compound, miss
	bysort gas: tab treatment large_compound
	*/
	
	sort Comp_id
	
//Add variables which will state the date and entry number of each line
	gen date="$S_DATE"
	gen entry=`num'


//Save important files

	//Save the matrix G and other matrices/vectors for later use
	mata
	mata matsave "data/safe/Matrices_`num'_$S_DATE" G 
	mata matsave "data/use/Matrices" G , replace
	end	
	
	//Save today's assignment with covariates and data
	save "data/safe/Daily_compoundassignment_`num'_$S_DATE"
	drop efficiency 
	save "data/view/Daily_Compoundassignment_`num'_$S_DATE"
	use "data/safe/Daily_compoundassignment_`num'_$S_DATE", clear
	
	//If no previous master exists (first round only), then create the master. Otherwise append to the old one. 
	capture confirm file "data\use\Compoundassignment.dta"
		if _rc==601 {
		save "data/safe/Master_compoundassignment_`num'_$S_DATE.dta"
		save "data/use/Compoundassignment.dta"
		drop efficiency 
		sort Comp_id
		save "data/view/Compoundassignment.dta"
		}
		
		else{
		append using "data/use/Compoundassignment.dta"
		sort Comp_id
		save "data/safe/Master_compoundassignment_`num'_$S_DATE.dta"
		save "data/use/Compoundassignment.dta", replace
		drop efficiency 
		sort Comp_id
		save "data/view/Compoundassignment.dta", replace
		}
//		




//Delete TodaysCovariates file from data/use folder (it's still in safe folder)
	forvalues n=1/100 {
		capture erase "data/use/TodaysCovariates_`n'.dta"
			if _rc==0 {
				continue, break
				}
		}
	//
		
	display "To view the compound assignment, type 'q' into the command line and press enter. When you are finished viewing, type 'end' and press enter. The assignment document can be viewed later by opening stata and opening the document."
	
	pause
	browse
	pause
	clear

/*	
************************************************************************
* END
************************************************************************
timer off 100
timer list
cap log close
exit
*/



