// atkinson-optimal-standalone-example.do
version 11

/* *********************************************************************
* PART 0: HOUSEKEEPING
********************************************************************* */
clear all
mata: mata clear
cap log close
set more off
set trace off
set linesize 140
set memory 200m
set matsize 1000
set scheme s2mono
set seed 12345679
timer clear

cap cd "C:\Users\Guiteras\Dropbox\Cholera-Disgust\Randomization"

local LOGNAME atkinson-optimal-standalone-example
log using logfiles/`LOGNAME', replace
pwd

timer on 100

/*
	atkinson-optimal-standalone-example.do

 	PART 0.A: INTRODUCTION AND DISCUSSION
	
	This is almost identical to the Efron biased coin approach
		so we skip the step of doing the second unit by hand
	
	descriptive statistics, tables and figures
	
	inputs: synthetic data

	output:	various tables and figures saved in
				/tables and /figures directories
				
	Version history:

		20110807	RPG		first version
		20110808	RPG		more systematic procedure for first units
							tiebreaker
		

	Directory structure
	
		base:				Dropbox/Cholera-Disgust/Randomization
		subdirectories:		/data			
							/data/raw		source data (not to be touched)
							/dofiles		contains this and other dofiles
							/logfiles		save log files here
							/figures		export figures (eps, pdf) here
							/tables			tables created here
												contains excel and pdf
							/tables/txt		export raw material for tables here
												contains txt and csv
*/	

/* ********************************************************************
* PART 0.B:	Define useful parameters (macros, scalars, etc), labels, etc
********************************************************************* */
	
/* *********************************************************************
* PART 0.C:	Define programs
********************************************************************* */
// programs here

/* *********************************************************************
* PART 0.D:	Sections to run / skip
********************************************************************* */


// create some fake data on compounds
local NUM_OBS				200
local SHARE_NOGAS			0.3
local COMPOUND_SIZE_L		6
local COMPOUND_SIZE_U		20
local NUM_TREATMENTS		4

set obs `NUM_OBS'
gen int obs_num = _n
// generate compound size (# of households)
// "To generate random integers over [a,b], use a+int((b-a+1)*runiform())."
gen num_hh = `COMPOUND_SIZE_L' + int((`COMPOUND_SIZE_U'-`COMPOUND_SIZE_L'+1)*runiform())
tab num_hh, miss
// create gas indicator
gen u_1 = runiform()
gen byte gas = u_1>=`SHARE_NOGAS'
tab gas, miss
drop u_1
order obs_num num_hh gas 

mata:

	// PART A: read in key parameters
	//	set up some parameters and other inputs that do not
	//	change from observation to observation

	// number of treatments (t in Atk notation)
	NUM_TREATMENTS = strtoreal(st_local("NUM_TREATMENTS"))
	NUM_TREATMENTS	
	
	// an identity matrix with one row per possible treatment
	POTENTIAL_TREATMENTS = I(NUM_TREATMENTS)
	POTENTIAL_TREATMENTS
	
	// get all the covariates -- this is a bit cheating since
	//	we won't have these data when implementing
	st_view(DATA=.,.,("num_hh","gas"))
	DATA[1..2,1..2]
	NUM_COVARS = cols(DATA)
	NUM_COVARS
	// note that NUM_COVARS = q-1 in Atkinson's notation
	
	// number of observations
	NUM_OBS = strtoreal(st_local("NUM_OBS"))
	
	// initialize variables that will contain index of max-var treatment
	ii = J(0,0,.)
	w  = J(0,0,.)

	// similarly, the matrix of contrasts (L' in Atk) and
	//	the A matrix (L augmented with a block of zeros)
	//	do not change from one observation to the next

	// L': matrix of contrasts
	L_prime_1 = J(NUM_TREATMENTS-1,1,1)
	L_prime_2 = I(NUM_TREATMENTS-1,NUM_TREATMENTS-1)
	L_prime = L_prime_1, (-1*L_prime_2)
	L_prime
	L = L_prime'
	L
	L'
	
	// A': augment with block of zeros to zero out the nuisance parameters
	A_prime_2 = J(NUM_TREATMENTS-1,NUM_COVARS,0)
	A_prime = L', A_prime_2
	A = A_prime'
	A
	A'
	
	// record dimensions of various important matrices
	num_cols_H = NUM_TREATMENTS
	num_cols_Z = NUM_COVARS
	num_cols_G = num_cols_H + num_cols_Z
	dim_GprimeG = num_cols_G						// GprimeG def'd below
	num_cols_H, num_cols_Z, num_cols_G, dim_GprimeG
	
	// need some noise to regularize information matrix for first t
	//	observations (see discussion below)
	epsilon = 0.001
	diagonal_epsilon = epsilon*I(dim_GprimeG,dim_GprimeG)
	
	// will store important data
	treatment_history 	= J(NUM_OBS,1,.)
	d_A_history 		= J(NUM_OBS,NUM_TREATMENTS,.)
	efficiency			= J(NUM_OBS,1,.)
	loss 				= J(NUM_OBS,1,.)
	
	// PART B: treatment assignment
	// **** B.1 FIRST OBSERVATION ****

	// matrix of treatment assignments H
	H = J(1,NUM_TREATMENTS,0)
	// assign first at random
	first_treatment = ceil((NUM_TREATMENTS-1+1)*runiform(1,1))
	H[1,first_treatment] = 1
	H
	
	treatment_history[1,1] = first_treatment
	
	// matrix of covariates Z
	Z = DATA[|1,1 \ 1,.|]
	Z
	
	G = H, Z		// this is G_n in Atk notation
	G
	

//	for (n=2; n<=10; n++) {
	for (n=2; n<=NUM_OBS; n++) {
		printf("Observation %f\n", n)
	
		GprimeG = G'*G
		// GprimeG
		// det(GprimeG)
		
		if (det(GprimeG)==0) {
			printf("GprimeG matrix is singular, adding epsilon noise\n")
			GprimeG = GprimeG + diagonal_epsilon
		}
		
		// holds values of generalized variance 
		d_A 	= J(1,NUM_TREATMENTS,.)
		
		z_nplus1 = (DATA[|n,1 \ n,.|])'

		for (j=1; j<=NUM_TREATMENTS; j++) {
			h_nplus1 = (POTENTIAL_TREATMENTS[j,.])'
			g_nplus1 = h_nplus1 \ z_nplus1
			// g_nplus1

			d_A_j = g_nplus1'*invsym(GprimeG)*A*invsym(A'*invsym(GprimeG)*A)*A'invsym(GprimeG)*g_nplus1
			// d_A_j
			d_A[1,j] = d_A_j
				
		}
		
		d_A_history[n,] = d_A
		printf("current value of d_A matrix")
		d_A
		
		// now assign probabilities to each of the treatments
		// 	based on Atkinson's optimal formula (Atkinson JRSS 2002 eqn 11)
		// probability weight is that treatment's variance,
		//	normalized by the sum of the variances
		sum_d_A = rowsum(d_A)
		printf("Sum of weights (variances) is %f", sum_d_A)
		DA_weight = d_A / sum_d_A
		printf("probability weights are")
		DA_weight
		
		// choose treatment via weighted randomization
		choice = rdiscrete(1, 1, DA_weight)
		printf("treatment chosen is %f\n", choice)
		h_nplus1 = J(4,1,0)
		h_nplus1[choice,1] = 1
		
		treatment_history[n,1] = choice
		
		g_nplus1 = h_nplus1 \ z_nplus1
		g_nplus1
		
		H = H \ (h_nplus1')
		Z = Z \ (z_nplus1')
		G = H, Z
		G
		
		efficiency_numerator = (NUM_TREATMENTS^NUM_TREATMENTS) / (n^(NUM_TREATMENTS-1))
		efficiency_denominator = det(A'*invsym(G'*G)*A)
		efficiency_exponent = (1 / (NUM_TREATMENTS-1))
		efficiency[n,1] = (efficiency_numerator / efficiency_denominator) ^ efficiency_exponent
		
		/*
		// note - it is also possible to calculate efficiency in one step using the
		//	formula below. however, for transparency, we calculate in steps above.
		//	the results should be identical up to machine precision / rounding error
		// the one-step formula is marginally faster
		
		efficiency[n,1] = ( ( (NUM_TREATMENTS^NUM_TREATMENTS) / (n^(NUM_TREATMENTS-1))) ///
			/ (det(A'*invsym(G'*G)*A)) ///
			) ///
			^ (1 / (NUM_TREATMENTS-1))
		*/
		
	}
	
	
	// add treatment, efficiency and loss histories to stata dataset
	treatment_index = st_addvar("byte","treatment_DA")
	st_store((1,rows(treatment_history)),treatment_index,treatment_history)
	efficiency_index = st_addvar("float","efficiency_DA")
	st_store((1,rows(efficiency)),efficiency_index,efficiency)
	
end


gen loss = _n * (1 - efficiency)

tab treatment
tab treatment gas, miss
summ num_hh, det
gen byte large_compound = num_hh>`r(p50)'
tab treatment large_compound, miss
bysort gas: tab treatment large_compound

sort obs_num
save atkinson-optimal-standalone-example, replace

	
************************************************************************
* END
************************************************************************
timer off 100
timer list
cap log close
exit
